@@ -1,4 +1,5 @@
 import os
+import typing
 import warnings
 from types import ModuleType
 from warnings import warn
@@ -64,19 +65,19 @@ def _eval_quiet(expr):
 # FIXME: should this be part of the API for rinterface ?
 #        (may be it is already the case and there is code
 #        duplicaton ?)
-def reval(string, envir=_globalenv):
+def reval(string: str,
+          envir: typing.Optional[rinterface.SexpEnvironment] = None):
     """ Evaluate a string as R code
     :param string: R code
     :type string: a :class:`str`
-    :param envir: an environment in which the environment should take
-      place (default: R's global environment)
+    :param envir: Optional environment to evaluate the R code.
     """
     p = rinterface.parse(string)
     res = _reval(p, envir=envir)
     return res
 
 
-def quiet_require(name, lib_loc=None):
+def quiet_require(name: str, lib_loc=None):
     """ Load an R package /quietly/ (suppressing messages to the console). """
     if lib_loc is None:
         lib_loc = "NULL"
@@ -164,8 +165,8 @@ class Package(ModuleType):
     _rpy2r = None
     _exported_names = None
     _symbol_r2python = None
-    __version__ = None
-    __rdata__ = None
+    __version__: typing.Optional[str] = None
+    __rdata__: typing.Optional[PackageData] = None
 
     def __init__(self, env, name, translation={},
                  exported_names=None, on_conflict='fail',
@@ -238,6 +239,7 @@ class Package(ModuleType):
                          exception)
         symbol_mapping.update(resolutions)
         reserved_pynames = set(dir(self))
+        cv = conversion.get_conversion()
         for rpyname, rnames in symbol_mapping.items():
             # last paranoid check
             if len(rnames) > 1:
@@ -257,9 +259,9 @@ class Package(ModuleType):
                 self._exported_names.add(rpyname)
             try:
                 riobj = self._env[rname]
-            except rinterface.RRuntimeError as rre:
+            except rinterface.embedded.RRuntimeError as rre:
                 warn(str(rre))
-            rpyobj = conversion.rpy2py(riobj)
+            rpyobj = cv.rpy2py(riobj)
             if hasattr(rpyobj, '__rname__'):
                 rpyobj.__rname__ = rname
             # TODO: shouldn't the original R name be also in the __dict__ ?
@@ -318,7 +320,7 @@ class InstalledSTPackage(SignatureTranslatedPackage):
             try:
                 doc.append(rhelp.docstring(self.__rname__,
                                            self.__rname__ + '-package',
-                                           sections=['description']))
+                                           sections=['\\description']))
             except rhelp.HelpNotFoundError:
                 doc.append('[R help was not found]')
         return os.linesep.join(doc)
@@ -346,7 +348,7 @@ class InstalledPackage(Package):
             try:
                 doc.append(rhelp.docstring(self.__rname__,
                                            self.__rname__ + '-package',
-                                           sections=['description']))
+                                           sections=['\\description']))
             except rhelp.HelpNotFoundError:
                 doc.append('[R help was not found]')
         return os.linesep.join(doc)
@@ -388,7 +390,7 @@ class InstalledPackages(object):
         self.colnames = self.lib_results.do_slot('dimnames')[1]  # column names
         self.lib_packname_i = self.colnames.index('Package')
 
-    def isinstalled(self, packagename):
+    def isinstalled(self, packagename: str):
         if not isinstance(packagename, rinterface.StrSexpVector):
             rinterface.StrSexpVector((packagename, ))
         else:
@@ -412,7 +414,7 @@ class InstalledPackages(object):
             yield tuple(lib_results[x*nrows+row_i] for x in colrg)
 
 
-def isinstalled(name,
+def isinstalled(name: str,
                 lib_loc=None):
     """
     Find whether an R package is installed
@@ -426,7 +428,7 @@ def isinstalled(name,
     return instapack.isinstalled(name)
 
 
-def importr(name,
+def importr(name: str,
             lib_loc=None,
             robject_translations={},
             signature_translation=True,
@@ -478,6 +480,7 @@ def importr(name,
                       **{'lib.loc': rinterface.StrSexpVector((lib_loc, ))})[0]
     if not ok:
         raise LibraryError("The R package %s could not be imported" % name)
+    exported_names: typing.Optional[typing.Set[str]]
     if _package_has_namespace(name,
                               _system_file(package=name)):
         env = _get_namespace(name)
@@ -488,6 +491,7 @@ def importr(name,
         exported_names = None
         version = None
 
+    pack: typing.Union[InstalledSTPackage, InstalledPackage]
     if signature_translation:
         pack = InstalledSTPackage(env, name,
                                   translation=robject_translations,
