@@ -91,6 +91,7 @@ def r_home_from_subprocess() -> Optional[str]:
 # TODO: move all Windows all code into an os-specific module ?
 def r_home_from_registry() -> Optional[str]:
     """Return the R home directory from the Windows Registry."""
+    from packaging.version import Version
     try:
         import winreg  # type: ignore
     except ImportError:
@@ -101,10 +102,27 @@ def r_home_from_registry() -> Optional[str]:
     # the for-loop breaks at the first hit.
     for w_hkey in [winreg.HKEY_CURRENT_USER, winreg.HKEY_LOCAL_MACHINE]:
         try:
-            with winreg.OpenKeyEx(w_hkey,
-                                  'Software\\R-core\\R',
-                                  0, winreg.KEY_QUERY_VALUE) as hkey:
-                r_home = winreg.QueryValueEx(hkey, 'InstallPath')[0]
+            with winreg.OpenKeyEx(w_hkey, 'Software\\R-core\\R') as hkey:
+
+                # >v4.x.x: grab the highest version installed
+                def get_version(i):
+                    try:
+                        return Version(winreg.EnumKey(hkey, i))
+                    except Exception:
+                        return None
+
+                latest = max(
+                    (v for v in (get_version(i)
+                                 for i in range(winreg.QueryInfoKey(hkey)[0]))
+                     if v is not None)
+                )
+
+                with winreg.OpenKeyEx(hkey, f'{latest}') as subkey:
+                    r_home = winreg.QueryValueEx(subkey, "InstallPath")[0]
+
+                # check for an earlier version
+                if not r_home:
+                    r_home = winreg.QueryValueEx(hkey, 'InstallPath')[0]
         except Exception:  # FileNotFoundError, WindowsError, OSError, etc.
             pass
         else:
